/*
 * set.h
 *
 *  Created on: 28. 4. 2021
 *      Author: mmacisak
 */

#ifndef SET_H_
#define SET_H_

void v_set_faults_reset(double val);
void v_set_state(double state);
double v_get_state(void);
void v_set_stop_set(unsigned char val);
void v_set_start_on(unsigned char val);
void v_set_commands_set(unsigned char val);
void v_set_dspace_reset(unsigned char val);
void set_duty(double val);

#endif /* SET_H_ */
